set ZUCCHERO;   		# dolcificanti naturali
set MISCELE;  		# tipi miscele
set DOLCIFICANTE_ARTIFICIALE; 	# dolcificanti artificiali

set DOLCIFICANTE := ZUCCHERO union DOLCIFICANTE_ARTIFICIALE;

param percentuale {DOLCIFICANTE,MISCELE} >= 0, <= 1; 	# percentuale del tipo di dolcificante nella miscela 
param prezzo {MISCELE} >= 0;    				# prezzo di una tonnellete della miscela 
param richiesta{ZUCCHERO} >= 0;  			# richiesta del tipo di zucchero
param quantita_totale:=sum{z in ZUCCHERO} richiesta[z]; # quantità totale di dolcificante da ottenere

param bonta {MISCELE} >= 0; 					# bontà della  varie miscele 

param acquisto_min {MISCELE} >= 0;  # quantità minime da acquistare per le varie miscele

param qualita;		# qualità finale richiesta

check {f in MISCELE}: sum {d in DOLCIFICANTE} percentuale[d,f] = 1;   # controllo sulla percentuale

var Qta {f in MISCELE} >= 0; # ton acquistate

minimize Costo: sum {f in MISCELE} prezzo[f] * Qta[f];  # Obiettivo:  minimizzare il costo di acquisto

subject to Totale_Zucchero: sum {f in MISCELE} Qta[f] = quantita_totale;	# vincolo sul totale da acquistare

subject to Richiesta_Finale {z in ZUCCHERO}:
   (sum {f in MISCELE} percentuale[z,f] * Qta[f])  = (richiesta[z] / quantita_totale) * (sum {zuc in ZUCCHERO} sum {f in MISCELE} percentuale[zuc,f] * Qta[f]);   # le percentuali finali di zucchero sono #fissate

subject to Qualita: (sum{f in MISCELE} bonta[f] * Qta[f])  >= qualita* (sum{f in MISCELE} Qta[f]);

subject to Contratti {f in MISCELE}: Qta[f] >= acquisto_min[f];  # bisogna acquistare almeno una certa quantità di ogni miscela
