model;
#definizione insiemi

set PERIODI ordered := 1 .. 12;
set TIPOPROD;	#tipologie produttive


#definizione parametri

param dom{PERIODI} >= 0;	# domanda prodotto per ogni periodo
param costoprod{TIPOPROD} > 0;	#costi per tipologia di produzione
param costooper >= 0;	#stipendio operaio
param costoass > 0;	#costo assunzione operai
param costolic > 0;	#costo licenziamento operai
param costomag > 0;	#costo unitario magazzino
param prodmax{TIPOPROD} > 0 default Infinity;	# produzione massima per tipologia
param costomanord := costooper/prodmax["ordinaria"];	#costo manodopera ordinaria
param capmag > 0 default Infinity;	# capacita massima del magazzino
param persI >= 0 integer;	# personale disponibile inizialmente
param persF >= 0 integer;	# personale richiesto per l'ultimo periodo  
param magI >= 0;	# unita di prodotto disponibile inizialmente in magazzino
param magF >= 0;	# unita di prodotto richiesto per l'ultimo periodo in magazzino


#definizione variabili

var Oper{PERIODI} >= 0 integer;	#numero operai presenti per periodo
var Ass{PERIODI} >= 0 integer;	#numero operai assunti per periodo
var Lic{PERIODI} >= 0 integer;	#numero operai licenziati per periodo
var Prod{PERIODI,TIPOPROD} >= 0;	# quantita prodotta per periodo per tipologia produttiva
var Mag{PERIODI} >= 0, <= capmag;	# quantita immagazzinata in magazzino per periodo


#definizione vincoli

subject to personale_finale:
	Oper[last(PERIODI)] = persF;

subject to magazzino_finale:
	Mag[last(PERIODI)] = magF;

subject to limite_produzione{t in PERIODI, p in TIPOPROD}:
	Prod[t,p] <= prodmax[p]*(if p == 'ordinaria' then Oper[t] else 1);

subject to bilancio_personale{t in PERIODI}:
(if t == first(PERIODI) then persI else Oper[prev(t,PERIODI)]) + Ass[t] = Oper[t] + Lic[t];

subject to bilancio_magazzino{t in PERIODI}:
	(if t== first(PERIODI) then magI else Mag[prev(t,PERIODI)]) + sum{p in TIPOPROD} Prod[t,p] = Mag[t] + dom[t];

subject to level {t in PERIODI: t > first(PERIODI)}:
	Prod[t,'ordinaria'] + Prod[t,'straordinaria'] = Prod[prev(t,PERIODI),'ordinaria'] + Prod[prev(t,PERIODI),'straordinaria'];
	
	
# definizione obiettivo

minimize costo_complessivo: 
	sum{t in PERIODI,p in TIPOPROD} (costoprod[p] - if p=='ordinaria' then costomanord else 0)*Prod[t,p] 
	+ sum{t in PERIODI}(costomag*Mag[t] + costooper*Oper[t]) + sum{t in PERIODI}(costoass*Ass[t] + costolic*Lic[t]);
	